import logging
from textwrap import dedent

from crewai import Agent

from mtmai.llm.llm import get_llm_chatbot_default

logger = logging.getLogger()


def step_callback(a):
    logger.info("调用 step_callback %s", a)


class JokAgents:
    def __init__(self):
        pass

    def joke_writer_agent(self):
        llm = get_llm_chatbot_default()

        return Agent(
            role="幽默段子写手",
            goal="创作幽默段子",
            backstory=dedent("""\
				作为一个网站的博主,你非常删除编写适合短视频社交平台的幽默段子， 作品容易吸引年轻人浏览，点击关注"""),
            verbose=True,
            allow_delegation=False,
            llm=llm,
            step_callback=step_callback,
        )

    # def email_action_agent(self):
    #     return Agent(
    #         role="Email Action Specialist",
    #         goal="Identify action-required emails and compile a list of their IDs",
    #         backstory=dedent("""\


# 			With a keen eye for detail and a knack for understanding context, you specialize
# 			in identifying emails that require immediate action. Your skill set includes interpreting
# 			the urgency and importance of an email based on its content and context."""),
#         tools=[
#             # GmailGetThread(api_resource=self.gmail.api_resource),
#             # TavilySearchResults(),
#         ],
#         verbose=True,
#         allow_delegation=False,
#     )

# def email_response_writer(self):
#     return Agent(
#         role="Email Response Writer",
#         goal="Draft responses to action-required emails",
#         backstory=dedent("""\
# 			You are a skilled writer, adept at crafting clear, concise, and effective email responses.
# 			Your strength lies in your ability to communicate effectively, ensuring that each response is
# 			tailored to address the specific needs and context of the email."""),
#         tools=[
#             # TavilySearchResults(),
#             # GmailGetThread(api_resource=self.gmail.api_resource),
#             CreateDraftTool.create_draft,
#         ],
#         verbose=True,
#         allow_delegation=False,
#     )



from datetime import datetime, timedelta

import jwt
from fastapi import Depends, HTTPException, status
from fastapi.security import OAuth2PasswordBearer
from jwt.exceptions import InvalidTokenError
from pydantic import ValidationError

# from mtmai.chainlit.config import config
# from mtmai.chainlit.oauth_providers import get_configured_oauth_providers
from mtmai.core import security
from mtmai.core.config import settings
from mtmai.core.logging import get_logger
from mtmai.crud import curd
from mtmai.deps import get_asession
from mtmai.models.models import TokenPayload, User

logger = get_logger()
reuseable_oauth = OAuth2PasswordBearer(tokenUrl="/login", auto_error=False)


def get_jwt_secret():
    return settings.SECRET_KEY


def ensure_jwt_secret():
    if require_login() and get_jwt_secret() is None:
        raise ValueError(
            "You must provide a JWT secret in the environment to use authentication. Run `chainlit create-secret` to generate one."
        )


def is_oauth_enabled():
    return config.code.oauth_callback and len(get_configured_oauth_providers()) > 0


def require_login():
    return True


# def get_configuration():
#     return {
#         "requireLogin": require_login(),
#         "passwordAuth": config.code.password_auth_callback is not None,
#         "headerAuth": config.code.header_auth_callback is not None,
#         "oauthProviders": get_configured_oauth_providers()
#         if is_oauth_enabled()
#         else [],
# }


def create_jwt(data: User) -> str:
    to_encode = data.to_dict()
    to_encode.update(
        {
            "exp": datetime.utcnow() + timedelta(minutes=60 * 24 * 15),  # 15 days
        }
    )
    encoded_jwt = jwt.encode(to_encode, get_jwt_secret(), algorithm="HS256")
    return encoded_jwt


async def authenticate_user(
    token: str = Depends(reuseable_oauth), session=Depends(get_asession)
):
    try:
        logger.info(f"secret: {settings.SECRET_KEY}, token: {token}, ")
        payload = jwt.decode(
            token, settings.SECRET_KEY, algorithms=[security.ALGORITHM]
        )
        token_data = TokenPayload(**payload)
    except (InvalidTokenError, ValidationError):
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Could not validate credentials",
        )
    # user = await session.get(User, token_data.sub)
    user = await curd.get_user_by_id(session=session, id=token_data.sub)
    if not user:
        raise HTTPException(status_code=404, detail="User not found")
    if not user.is_active:
        raise HTTPException(status_code=400, detail="Inactive user")
    return user


async def get_current_user(
    token: str = Depends(reuseable_oauth), session=Depends(get_asession)
):
    if not require_login():
        return None
    logger.info(f"call get_current_user token: {token}")
    return await authenticate_user(token, session=session)



from langchain_core.prompts import ChatPromptTemplate
from langchain_core.runnables import RunnableConfig

from mtmai.agents.ctx import mtmai_context
from mtmai.agents.opencanvas.opencanvas_state import OpenCanvasState

WORK_DIR_NAME = "project"
WORK_DIR = f"/home/{WORK_DIR_NAME}"
MODIFICATIONS_TAG_NAME = "bolt_file_modifications"


class RespondToQueryNode:
    async def __call__(self, state: OpenCanvasState, config: RunnableConfig):
        user_id = state.userId
        params = state.params
        messages = state.messages
        assistant_prompt = ChatPromptTemplate.from_messages(
            [
                (
                    "system",
                    """You are an AI assistant tasked with responding to the users question.

The user has generated artifacts in the past. Use the following artifacts as context when responding to the users question.

You also have the following reflections on style guidelines and general memories/facts about the user to use when generating your response.
<reflections>
{reflections}
</reflections>

<artifacts>
{artifacts}
</artifacts>""",
                ),
                ("placeholder", "{messages}"),
            ]
        ).partial(
            reflections="",
            artifacts="",
            messages=messages,
        )
        aiMsg = await mtmai_context.ainvoke_model(assistant_prompt, state, tools=[])
        return {"messages": aiMsg}



from phi.agent import Agent
from phi.tools.yfinance import YFinanceTools

from mtmai.agents.phiagents.config import model
from mtmai.core.logging import get_logger

logger = get_logger()
finance_agent = Agent(
    name="Finance Agent",
    role="Get financial data",
    model=model,
    tools=[
        YFinanceTools(
            stock_price=True,
            analyst_recommendations=True,
            company_info=True,
            company_news=True,
        )
    ],
    instructions=["Always use tables to display data"],
    markdown=True,
    show_tool_calls=True,
)



# defines how core data types in autogenstudio are serialized and stored in the database

from datetime import datetime
from enum import Enum
from typing import List, Optional, Union
from uuid import UUID, uuid4

from autogen_core import ComponentModel
from sqlalchemy import ForeignKey, Integer
from sqlmodel import JSON, Column, DateTime, Field, SQLModel, func

from .ag import MessageConfig, MessageMeta, TeamResult


class Team(SQLModel, table=True):
    __table_args__ = {"sqlite_autoincrement": True}
    id: Optional[int] = Field(default=None, primary_key=True)
    created_at: datetime = Field(
        default_factory=datetime.now,
        sa_column=Column(DateTime(timezone=True), server_default=func.now()),
    )  # pylint: disable=not-callable
    updated_at: datetime = Field(
        default_factory=datetime.now,
        sa_column=Column(DateTime(timezone=True), onupdate=func.now()),
    )  # pylint: disable=not-callable
    user_id: Optional[str] = None
    version: Optional[str] = "0.0.1"
    component: Union[ComponentModel, dict] = Field(sa_column=Column(JSON))


class Message(SQLModel, table=True):
    __table_args__ = {"sqlite_autoincrement": True}
    id: Optional[int] = Field(default=None, primary_key=True)
    created_at: datetime = Field(
        default_factory=datetime.now,
        sa_column=Column(DateTime(timezone=True), server_default=func.now()),
    )  # pylint: disable=not-callable
    updated_at: datetime = Field(
        default_factory=datetime.now,
        sa_column=Column(DateTime(timezone=True), onupdate=func.now()),
    )  # pylint: disable=not-callable
    user_id: Optional[str] = None
    version: Optional[str] = "0.0.1"
    config: Union[MessageConfig, dict] = Field(
        default_factory=MessageConfig, sa_column=Column(JSON)
    )
    session_id: Optional[int] = Field(
        default=None,
        sa_column=Column(Integer, ForeignKey("session.id", ondelete="CASCADE")),
    )
    run_id: Optional[UUID] = Field(default=None, foreign_key="run.id")
    message_meta: Optional[Union[MessageMeta, dict]] = Field(
        default={}, sa_column=Column(JSON)
    )


class Session(SQLModel, table=True):
    __table_args__ = {"sqlite_autoincrement": True}
    id: Optional[int] = Field(default=None, primary_key=True)
    created_at: datetime = Field(
        default_factory=datetime.now,
        sa_column=Column(DateTime(timezone=True), server_default=func.now()),
    )  # pylint: disable=not-callable
    updated_at: datetime = Field(
        default_factory=datetime.now,
        sa_column=Column(DateTime(timezone=True), onupdate=func.now()),
    )  # pylint: disable=not-callable
    user_id: Optional[str] = None
    version: Optional[str] = "0.0.1"
    team_id: Optional[int] = Field(
        default=None,
        sa_column=Column(Integer, ForeignKey("team.id", ondelete="CASCADE")),
    )
    name: Optional[str] = None


class RunStatus(str, Enum):
    CREATED = "created"
    ACTIVE = "active"
    COMPLETE = "complete"
    ERROR = "error"
    STOPPED = "stopped"


class Run(SQLModel, table=True):
    """Represents a single execution run within a session"""

    __table_args__ = {"sqlite_autoincrement": True}

    id: UUID = Field(default_factory=uuid4, primary_key=True, index=True)
    created_at: datetime = Field(
        default_factory=datetime.now,
        sa_column=Column(DateTime(timezone=True), server_default=func.now()),
    )
    updated_at: datetime = Field(
        default_factory=datetime.now,
        sa_column=Column(DateTime(timezone=True), onupdate=func.now()),
    )
    session_id: Optional[int] = Field(
        default=None,
        sa_column=Column(
            Integer, ForeignKey("session.id", ondelete="CASCADE"), nullable=False
        ),
    )
    status: RunStatus = Field(default=RunStatus.CREATED)

    # Store the original user task
    task: Union[MessageConfig, dict] = Field(
        default_factory=MessageConfig, sa_column=Column(JSON)
    )

    # Store TeamResult which contains TaskResult
    team_result: Union[TeamResult, dict] = Field(default=None, sa_column=Column(JSON))

    error_message: Optional[str] = None
    version: Optional[str] = "0.0.1"
    messages: Union[List[Message], List[dict]] = Field(
        default_factory=list, sa_column=Column(JSON)
    )

    class Config:
        json_encoders = {UUID: str, datetime: lambda v: v.isoformat()}



from calendar import day_name

import structlog
from langchain_core.prompts import ChatPromptTemplate
from langchain_core.runnables import RunnableConfig

from mtmai.agents.ctx import mtmai_context  # noqa
from mtmai.models.graph_config import HomeChatState

LOG = structlog.get_logger()



class CreateTaskNode:
    def __init__(self):
        pass

    async def __call__(self, state: HomeChatState, config: RunnableConfig):
        user_id = mtmai_context.user_id
        assistant_prompt = ChatPromptTemplate.from_messages(
            [
                (
                    "system",
                    """<background_story>
你是很有用的助手，擅长根据用户的一段文字，生成结构化的任务模板
</background_story>
{workbench_prompt}
<system_constraints></system_constraints>

<code_formatting_info>
必须使用JSON输出数据
</code_formatting_info>

<message_formatting_info>
</message_formatting_info>

<examples>
    <example>
        <input>
        我要生成文章
        </input>
        <output>
        {{
            "task_type": "articleGen",
            "task_config": {{
                "title": "文章标题",
                "content": "文章内容"
            }}
        }}
        </output>
    </example>
</examples>
{additional_instructions}
""",
                ),
                ("placeholder", "{messages}"),
            ]
        ).partial(
            # tag_names=day_name,
            # MODIFICATIONS_TAG_NAME=MODIFICATIONS_TAG_NAME,
            # cwd="",
            workbench_prompt="",
            # siteId=params.get("siteId", None),
            userId=user_id,
            additional_instructions="",
        )
        ai_msg = await mtmai_context.ainvoke_model(
            assistant_prompt,
            state,
            # tools=primary_assistant_tools
        )
        return {
            "messages": ai_msg,
            # "next": "human",
        }



import logging
from textwrap import dedent
from typing import Annotated

import httpx
from langchain_core.tools import tool
from langgraph.prebuilt import InjectedState

from mtmai.core.config import settings

logger = logging.getLogger()

search_result_limit = 1


@tool(response_format="content_and_artifact")
def search_tool(query: str, state: Annotated[dict, InjectedState]):  # noqa: ARG001
    """Useful to search content from web."""
    searxng_url = f"{settings.SEARXNG_URL_BASE}/search"
    logger.info("调用 search ( %s ), %s", searxng_url, query)

    with httpx.Client() as client:
        params = {"q": query, "format": "json"}
        r = client.get(searxng_url, params=params)
        # r.raise_for_status()

        if r.status_code == 429:
            return ("大模型调用每分钟token数量限制, 请在1分钟后重试", {})
        r.raise_for_status()

        search_results = r.json()

        result_list2 = search_results.get("results", [])[:search_result_limit]

        result_list3 = [
            {
                "title": x.get("title", ""),
                "url": x.get("url", ""),
                "content": x.get("content", ""),
            }
            for x in result_list2
        ]

        content_lines = ["搜索结果:"]
        for x in result_list3:
            content_lines.append(
                dedent(f"""title: {x.get("title")}
                      content: {x.get("content")}
                      """)
            )

        return (
            "\n".join(content_lines),
            {
                "artifaceType": "ArtifactSearchResults",
                "props": {
                    "title": f"{query}的搜索结果",
                    "results": result_list3,
                    "suggestions": search_results.get("suggestions", []),
                    # "infoboxes": search_results.get("infoboxes", []),
                },
            },
        )



import urllib
import uuid

import aiofiles
import mistune


async def write_to_file(filename: str, text: str) -> None:
    """Asynchronously write text to a file in UTF-8 encoding.

    Args:
        filename (str): The filename to write to.
        text (str): The text to write.
    """
    # Convert text to UTF-8, replacing any problematic characters
    text_utf8 = text.encode("utf-8", errors="replace").decode("utf-8")

    async with aiofiles.open(filename, "w", encoding="utf-8") as file:
        await file.write(text_utf8)


async def write_text_to_md(text: str, path: str) -> str:
    """Writes text to a Markdown file and returns the file path.

    Args:
        text (str): Text to write to the Markdown file.

    Returns
    -------
        str: The file path of the generated Markdown file.
    """
    task = uuid.uuid4().hex
    file_path = f"{path}/{task}.md"
    await write_to_file(file_path, text)
    print(f"Report written to {file_path}")
    return file_path


async def write_md_to_pdf(text: str, path: str) -> str:
    """Converts Markdown text to a PDF file and returns the file path.

    Args:
        text (str): Markdown text to convert.

    Returns
    -------
        str: The encoded file path of the generated PDF.
    """
    task = uuid.uuid4().hex
    file_path = f"{path}/{task}.pdf"

    try:
        # Moved imports to inner function to avoid known import errors with gobject-2.0
        from md2pdf.core import md2pdf

        md2pdf(
            file_path,
            md_content=text,
            # md_file_path=f"{file_path}.md",
            css_file_path="./mtmai/multi_agents/agents/utils/pdf_styles.css",  # Updated path
            base_url=None,
        )
        print(f"Report written to {file_path}")
    except Exception as e:
        print(f"Error in converting Markdown to PDF: {e}")
        return ""

    encoded_file_path = urllib.parse.quote(file_path)
    return encoded_file_path


async def write_md_to_word(text: str, path: str) -> str:
    """Converts Markdown text to a DOCX file and returns the file path.

    Args:
        text (str): Markdown text to convert.

    Returns
    -------
        str: The encoded file path of the generated DOCX.
    """
    task = uuid.uuid4().hex
    file_path = f"{path}/{task}.docx"

    try:
        from docx import Document
        from htmldocx import HtmlToDocx

        # Convert report markdown to HTML
        html = mistune.html(text)
        # Create a document object
        doc = Document()
        # Convert the html generated from the report to document format
        HtmlToDocx().add_html_to_document(html, doc)

        # Saving the docx document to file_path
        doc.save(file_path)

        print(f"Report written to {file_path}")

        encoded_file_path = urllib.parse.quote(f"{file_path}.docx")
        return encoded_file_path

    except Exception as e:
        print(f"Error in converting Markdown to DOCX: {e}")
        return ""



# /api/runs routes
from typing import Dict
from uuid import UUID

from fastapi import APIRouter, Depends, HTTPException
from pydantic import BaseModel

from ..deps import get_db
from ..models.ag_db import Message, Run

router = APIRouter()


class CreateRunRequest(BaseModel):
    session_id: int
    user_id: str


# @router.post("/")
# async def create_run(
#     request: CreateRunRequest,
#     db=Depends(get_db),
# ) -> Dict:
#     """Create a new run with initial state"""
#     session_response = db.get(
#         Session,
#         filters={"id": request.session_id, "user_id": request.user_id},
#         return_json=False,
#     )
#     if not session_response.status or not session_response.data:
#         raise HTTPException(status_code=404, detail="Session not found")

#     try:
#         # Create run with default state
#         run = db.upsert(
#             Run(
#                 session_id=request.session_id,
#                 status=RunStatus.CREATED,
#                 task=None,  # Will be set when run starts
#                 team_result=None,
#             ),
#             return_json=False,
#         )
#         return {"status": run.status, "data": {"run_id": str(run.data.id)}}
#     except Exception as e:
#         raise HTTPException(status_code=500, detail=str(e)) from e


# We might want to add these endpoints:


@router.get("/{run_id}")
async def get_run(run_id: UUID, db=Depends(get_db)) -> Dict:
    """Get run details including task and result"""
    run = db.get(Run, filters={"id": run_id}, return_json=False)
    if not run.status or not run.data:
        raise HTTPException(status_code=404, detail="Run not found")

    return {"status": True, "data": run.data[0]}


@router.get("/{run_id}/messages")
async def get_run_messages(run_id: UUID, db=Depends(get_db)) -> Dict:
    """Get all messages for a run"""
    messages = db.get(
        Message, filters={"run_id": run_id}, order="created_at asc", return_json=False
    )

    return {"status": True, "data": messages.data}



import logging

from langgraph.graph import StateGraph
from opentelemetry import trace
from typing_extensions import TypedDict


logger = logging.getLogger()

tracer = trace.get_tracer_provider().get_tracer(__name__)
logger = logging.getLogger()


class StateImage2Text(TypedDict):
    model_name: str | None = None
    user_prompt: str | None = None


def entry_text2image():
    pass


def node_text2image_gen():
    pass


class SubGraphText2Image:
    def __init__(self):
        pass

    @property
    def name(self):
        return "graphchatdemo"

    def create_graph(self):
        # nodes = Nodes()

        wf = StateGraph(StateImage2Text)
        wf.add_node("entry_text2image", entry_text2image)
        wf.add_node("node_text2image_gen", node_text2image_gen)
        wf.set_entry_point("entry_text2image")
        wf.add_edge("entry_text2image", "node_text2image_gen")
        # wf.add_edge("chat", "uidelta_node")
        # wf.add_edge("uidelta_node", "chat")
        # wf.add_conditional_edges(
        #     "chat",
        #     should_continue,
        #     {
        #         "tool_call": "tool_call",
        #         "ask_human": "uidelta_node",
        #         "end": END,
        #     },
        # )
        return wf
